// Analog_Input_Scan.cpp : Defines the entry point for the console application.
//
// HSDAQ data logger folder structure
//
//    +----\log
//          +----\xx.xx.xx.xx 
//                 +----\yyyy_mm_dd_hh_mm []
//                        +----xx_xx_xx_xx_4c42_yyyy_mm_dd_hh_mm_ss.bin [file name] 
//   
//    Note:
//       1.xx.xx.xx.xx : [The folder name is IP address of the PET-AR400]
//       2.yyyy_mm_dd_hh_mm [It's the default folder format yyyy:year, mm:month, dd:day, hh:hour, mm:minute]
//       3.10_1_107_125_4c42_2018_11_30_14_16_29.bin [file name]  
#include "stdafx.h"
#include "HSDAQ.h"
#include <stdio.h>
#include <conio.h>


int ReplaceChar(char *srcStr,char r, char t)
{
	int len=strlen(srcStr);
	int i;
	if(len)
	{
		for(i=0;i<len;i++)
		{
			if(srcStr[i]==r)
				srcStr[i]=t;
		}	
	}
	return 0;
}

unsigned WStrlen(LPTSTR wideStr)
{
	unsigned length = WideCharToMultiByte(CP_ACP, 0, wideStr, -1, 0, 0, NULL, NULL);
	return length;
}

void WideToChar(LPTSTR wideStr, char cStr[])
{
	unsigned length = WStrlen(wideStr);
	char* output = new char[length];
	WideCharToMultiByte(CP_ACP, 0, wideStr, -1, output , length,NULL,NULL);
	strcpy(cStr, output);
	delete[] output;
}
 void CharToWide(LPCSTR astr, LPTSTR wstr)
{
    MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, astr, -1, wstr, 2048);
}


int _tmain(int argc, _TCHAR* argv[])
{
	HANDLE hHS;
	char IPadd[64]="\0";
	char sdk_version[16]={0};
	char fw_version[32]={0};

	bool ret= false;
	short chCnt=0;
	short  useGain=0;


	short extriggMode=0;
	bool continuousMode=false;
	long continuousTimeout=0;
	unsigned long targetCnt=0;
	long sampleRate=0;

	short DataTransMethod=0;
	short AutoRun=0;

	if(argc>1)
	{
		wprintf(TEXT("%s\r\n"),argv[1]);
   
		WideCharToMultiByte(CP_ACP,   0,   argv[1],   -1,   IPadd,  64,   NULL,   NULL);
		printf("program start %s\r\n",IPadd);
	}
	else
	{
		printf("error argument\r\n");
		return 0;
	}

	char tmp[128]={0};
	
	printf("PET-AR400 Analog Input data logger & read log file example [N sample mode & Software trigger]\r\n");
	HS_GetSDKVersion(sdk_version);
	printf("HSDAQ SDK Ver=%s\r\n",sdk_version);
	sprintf(tmp,"%s",IPadd);

	//Step 1: Create a TCP connection with PET-AR400
	hHS = HS_Device_Create(tmp);
	if(hHS!=NULL)
	{
		HS_GetFirmwareVersion(hHS,fw_version);
		printf("PET-AR400 Firmware Ver=%s\r\n",fw_version);
		
		chCnt=4;
		extriggMode=0;
		targetCnt=400;
		sampleRate=1000; 
		ret=HS_SetAIScanParam(hHS, chCnt, useGain, extriggMode, sampleRate, targetCnt, DataTransMethod,AutoRun);
		//Step 2: SetScanParam sets chCnt, useGain, triggerMode, sampleRate, targetCnt to PET-AR400
		if(ret==false)
		{
			printf("Error code 0x%x\r\n",HS_GetLastError());
		}

		HS_GetAIScanParam(hHS, &chCnt, &useGain, &extriggMode, &sampleRate, &targetCnt,&DataTransMethod,&AutoRun); 
		//Step 3: GetScanParam getsx chCnt, useGain, triggerMode, sampleRate, targetCnt to check with PET-AR400
		printf("PET-AR400 first Scan parameters chCnt %d, useGain %d, triggMode %d, sampleRate %ld, targetCnt %lu \n", chCnt, useGain, extriggMode, sampleRate, targetCnt);

		Sleep(10);

		TCHAR tcfullPath[MAX_PATH];
		char  cfullPath[MAX_PATH];
		GetCurrentDirectory( MAX_PATH, tcfullPath );
		WideToChar(tcfullPath,cfullPath); 

		//Step 4: Start AI data logger
		ret=HS_StartLogger(hHS,cfullPath,2,0); // 2 mins
		//second paramter ==> NULL, it means the log file will saved to the current directory
		if(ret==false)
		{
			printf("Error code 0x%x\r\n",HS_GetLastError());
		}

		WORD BufferStatus=0;
		#define BUFFERSIZE 1000
		float fdataBuffer[BUFFERSIZE];
		unsigned long ulleng=0;
		unsigned long totalRecv=0;
		unsigned int remChannel=0;

		unsigned long CH1checkerror=0;
		unsigned long CH7checkerror=0;
		unsigned long otherCHcheckerror=0;
		bool quit=false;
		unsigned int triggerStatus;

		//Step 5: while loop to get total received data count from PET-AR400
		while(quit==false)
		{				
			ret=HS_GetTotalSamplingStatus(hHS,&ulleng,&triggerStatus);
			if(ret==false)
			{
				printf("Error code 0x%x\r\n",HS_GetLastError());
			}
			else
			{
				if(ulleng>=targetCnt && targetCnt>0) //N sample mode
				{					
					//Step 6:When the total number of samples reaches the target counts. Send a command to get the all acquisition data over the Ethernet.
					//ret=HS_TransmitDataCmd(hHS);
					Sleep(6000);
					break;
				}
				else
					printf(".");
			} 
			if(_kbhit())
			{
				Sleep(1);
				if(_getch()=='q' || _getch()=='Q')
					break;
			}						
			else
				Sleep(100);
		}
		//Step7: Stop AI data logger
		HS_StopLogger(hHS);
		printf("AP total read %lu\n", ulleng);
		//Step8: Close the connection
		HS_Device_Release(hHS);

		ReplaceChar(IPadd,'.','_');
		strcat(cfullPath,"\\log\\");
		strcat(cfullPath,IPadd);
		printf("\r\npath=%s\r\n",cfullPath);
		CharToWide(cfullPath,tcfullPath);

		
		short gchCnt=0;
		short guseGain=0;
		short gextriggMode=0;
		bool gcontinuousMode=false;
		long gcontinuousTimeout=0;
		unsigned long gtargetCnt=0;
		long gsampleRate=0;
		short gDataTransMethod;
		short gSyncMode;
		unsigned short gVarr[4];
		short oVarr[4];
		//Step 9: Get the total number of files in the specified folder according to the file type
		int ind=HS_GetAllLogFilesW(tcfullPath,0);
		printf("ind=%d\r\n",ind);
		if(ind>0)
		{
			for(int i=0;i<ind;i++)
			{
				TCHAR tcgetfulfilelPath[MAX_PATH]={0};
				char FileName[MAX_PATH]={0};
				
				//Step 10: Open log file by the index number that indicates the order in which the file is located in the specified folder
				HANDLE hlf=HS_LogFile_Open_byIndexW(i,tcgetfulfilelPath);
				wprintf(TEXT("%d ==>%s\r\n"),i,tcgetfulfilelPath);
				//Step 11:Get chCnt, useGain, triggerMode, sampleRate, targetCnt parameters of the selected log file
				HS_GetLogFile_AIScanConfigInfo(hlf,&gchCnt,&guseGain,&gextriggMode,&gsampleRate,&gDataTransMethod,&gSyncMode);
				printf("%d, %d, %d, %ld, %d ,%d\r\n",gchCnt,guseGain,gextriggMode,gsampleRate,gDataTransMethod,gSyncMode);
				
				unsigned short gainVal=0;
				short offsetVal=0;
				//Step 12:Get gain/offset parameters of the selected log file
				for(int i=0;i<gchCnt;i++)
				{
					HS_GetLogFile_GainOffset(hlf,i,guseGain, &gVarr[i], &oVarr[i]);
					printf("%d, %d, %d\r\n",i,gVarr[i],oVarr[i]);
				}
				DWORD filesize;
				int filetype;
				int fileversion;
				//Step 13:Get the file information of the selected log file
				HS_GetLogFileInfo(hlf,FileName,&filesize,&filetype,&fileversion);
				printf("%ld, %d\r\n",filesize,filetype);
				DWORD samplecount;
				char startdate[32],starttime[32];
				//Step 14:Get the total sampling counts and the date/time of the first data acquiring in the selected log file
				HS_GetLogFile_AIScanSampleInfo(hlf,&samplecount,startdate,starttime);
				printf("%ld, %s %s\r\n",samplecount,startdate,starttime);
				long *databuff=(long *)malloc(sizeof(long)*samplecount);
				//Step 15:Get the total data of the selected log file
				HS_GetLogFile_AIDataHex(hlf,0,samplecount,databuff);
				for(int aa=0;aa<samplecount;aa++)
				{
					
					int sch = aa % gchCnt;
					float fdata;
					HS_Calibrate_Data_Float(hlf,sch,guseGain,databuff[aa],&fdata);
					if (sch == (gchCnt-1)) //new line
                    {
						printf("%f\r\n",fdata);
                    }
                    else
                    {
						printf("%f,",fdata);    
                    }
					
				}
				
				//Step 16: Close the selected log file
				HS_LogFile_Close(hlf);
			}
		}

	}
	return 0;

}

